@@ -1,10 +1,9 @@
 package Net::DNS::Header;
 
-#
-# $Id: Header.pm 1381 2015-08-25 07:36:09Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1381 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Header.pm 1886 2022-12-20 11:27:25Z willem $)[2];
 
 
 =head1 NAME
@@ -15,7 +14,7 @@ Net::DNS::Header - DNS packet header
 
     use Net::DNS;
 
-    $packet = new Net::DNS::Packet;
+    $packet = Net::DNS::Packet->new();
     $header = $packet->header;
 
 
@@ -26,11 +25,10 @@ C<Net::DNS::Header> represents the header portion of a DNS packet.
 =cut
 
 
-use strict;
 use integer;
 use Carp;
 
-use Net::DNS::Parameters;
+use Net::DNS::Parameters qw(:opcode :rcode);
 
 
 =head1 METHODS
@@ -38,7 +36,7 @@ use Net::DNS::Parameters;
 
 =head2 $packet->header
 
-    $packet = new Net::DNS::Packet;
+    $packet = Net::DNS::Packet->new();
     $header = $packet->header;
 
 Net::DNS::Header objects emanate from the Net::DNS::Packet header()
@@ -69,7 +67,7 @@ sub string {
 	my $ns	   = $self->nscount;
 	my $ar	   = $self->arcount;
 
-	my $opt = $$self->edns;
+	my $opt	 = $$self->edns;
 	my $edns = $opt->_specified ? $opt->string : '';
 
 	return <<END . $edns if $opcode eq 'UPDATE';
@@ -105,7 +103,10 @@ Prints the string representation of the packet header.
 
 =cut
 
-sub print { print &string; }
+sub print {
+	print &string;
+	return;
+}
 
 
 =head2 id
@@ -119,11 +120,16 @@ A random value is assigned if the argument value is undefined.
 
 =cut
 
+my ( $cache1, $cache2, $limit );				# two layer cache
+
 sub id {
-	my $self = shift;
-	$$self->{id} = shift if scalar @_;
-	return $$self->{id} if defined $$self->{id};
-	$$self->{id} = int rand(0xffff);
+	my $self  = shift;
+	my $ident = scalar(@_) ? ( $$self->{id} = shift ) : $$self->{id};
+	return $ident if $ident;
+	$ident = int rand(0xffff);				# preserve short-term uniqueness
+	$ident = int rand(0xffff) while $cache1->{$ident}++ + exists( $cache2->{$ident} );
+	( $cache1, $cache2, $limit ) = ( {0 => 1}, $cache1, 50 ) unless $limit--;
+	return $$self->{id} = $ident;
 }
 
 
@@ -137,13 +143,14 @@ Gets or sets the query opcode (the purpose of the query).
 =cut
 
 sub opcode {
-	my $self = shift;
+	my ( $self, $arg ) = @_;
+	my $opcode;
 	for ( $$self->{status} ) {
-		return opcodebyval( ( $_ >> 11 ) & 0x0f ) unless scalar @_;
-		my $opcode = opcodebyname(shift);
-		$_ = ( $_ & 0x87ff ) | ( $opcode << 11 );
-		return $opcode;
+		return opcodebyval( ( $_ >> 11 ) & 0x0f ) unless defined $arg;
+		$opcode = opcodebyname($arg);
+		$_	= ( $_ & 0x87ff ) | ( $opcode << 11 );
 	}
+	return $opcode;
 }
 
 
@@ -157,23 +164,21 @@ Gets or sets the query response code (the status of the query).
 =cut
 
 sub rcode {
-	my $self = shift;
+	my ( $self, $arg ) = @_;
+	my $rcode;
 	for ( $$self->{status} ) {
-		my $arg = shift;
 		my $opt = $$self->edns;
 		unless ( defined $arg ) {
-			my $rcode = $opt->rcode;
-			return rcodebyval( $_ & 0x0f ) unless $opt->_specified;
-			$rcode = ( $rcode & 0xff0 ) | ( $_ & 0x00f );
+			$rcode = ( $opt->rcode & 0xff0 ) | ( $_ & 0x00f );
 			$opt->rcode($rcode);			# write back full 12-bit rcode
 			return $rcode == 16 ? 'BADVERS' : rcodebyval($rcode);
 		}
-		my $rcode = rcodebyname($arg);
+		$rcode = rcodebyname($arg);
 		$opt->rcode($rcode);				# full 12-bit rcode
 		$_ &= 0xfff0;					# low 4-bit rcode
 		$_ |= ( $rcode & 0x000f );
-		return $rcode;
 	}
+	return $rcode;
 }
 
 
@@ -187,13 +192,13 @@ Gets or sets the query response flag.
 =cut
 
 sub qr {
-	shift->_dnsflag( 0x8000, @_ );
+	return shift->_dnsflag( 0x8000, @_ );
 }
 
 
 =head2 aa
 
-    print "answer is ", $packet->header->aa ? "" : "non-", "authoritative\n";
+    print "response is ", $packet->header->aa ? "" : "non-", "authoritative\n";
     $packet->header->aa(0);
 
 Gets or sets the authoritative answer flag.
@@ -201,7 +206,7 @@ Gets or sets the authoritative answer flag.
 =cut
 
 sub aa {
-	shift->_dnsflag( 0x0400, @_ );
+	return shift->_dnsflag( 0x0400, @_ );
 }
 
 
@@ -215,7 +220,7 @@ Gets or sets the truncated packet flag.
 =cut
 
 sub tc {
-	shift->_dnsflag( 0x0200, @_ );
+	return shift->_dnsflag( 0x0200, @_ );
 }
 
 
@@ -229,7 +234,7 @@ Gets or sets the recursion desired flag.
 =cut
 
 sub rd {
-	shift->_dnsflag( 0x0100, @_ );
+	return shift->_dnsflag( 0x0100, @_ );
 }
 
 
@@ -243,7 +248,7 @@ Gets or sets the recursion available flag.
 =cut
 
 sub ra {
-	shift->_dnsflag( 0x0080, @_ );
+	return shift->_dnsflag( 0x0080, @_ );
 }
 
 
@@ -254,24 +259,24 @@ Unassigned bit, should always be zero.
 =cut
 
 sub z {
-	shift->_dnsflag( 0x0040, @_ );
+	return shift->_dnsflag( 0x0040, @_ );
 }
 
 
 =head2 ad
 
-    print "The result has ", $packet->header->ad ? "" : "not", "been verified\n";
+    print "The response has ", $packet->header->ad ? "" : "not", "been verified\n";
 
 Relevant in DNSSEC context.
 
-(The AD bit is only set on answers where signatures have been
+(The AD bit is only set on a response where signatures have been
 cryptographically verified or the server is authoritative for the data
 and is allowed to set the bit by policy.)
 
 =cut
 
 sub ad {
-	shift->_dnsflag( 0x0020, @_ );
+	return shift->_dnsflag( 0x0020, @_ );
 }
 
 
@@ -285,7 +290,7 @@ Gets or sets the checking disabled flag.
 =cut
 
 sub cd {
-	shift->_dnsflag( 0x0010, @_ );
+	return shift->_dnsflag( 0x0010, @_ );
 }
 
 
@@ -299,12 +304,13 @@ to the number of RRs in the zone section.
 
 =cut
 
-use vars qw($warned);
+our $warned;
 
 sub qdcount {
 	my $self = shift;
 	return $$self->{count}[0] || scalar @{$$self->{question}} unless scalar @_;
-	carp 'header->qdcount attribute is read-only' unless $warned++;
+	carp 'packet->header->qdcount attribute is read-only'	  unless $warned++;
+	return;
 }
 
 
@@ -323,7 +329,8 @@ to the number of RRs in the prerequisite section.
 sub ancount {
 	my $self = shift;
 	return $$self->{count}[1] || scalar @{$$self->{answer}} unless scalar @_;
-	carp 'header->ancount attribute is read-only' unless $warned++;
+	carp 'packet->header->ancount attribute is read-only'	unless $warned++;
+	return;
 }
 
 
@@ -342,7 +349,8 @@ to the number of RRs in the update section.
 sub nscount {
 	my $self = shift;
 	return $$self->{count}[2] || scalar @{$$self->{authority}} unless scalar @_;
-	carp 'header->nscount attribute is read-only' unless $warned++;
+	carp 'packet->header->nscount attribute is read-only'	   unless $warned++;
+	return;
 }
 
 
@@ -360,13 +368,14 @@ In dynamic update packets, this field is known as C<adcount>.
 sub arcount {
 	my $self = shift;
 	return $$self->{count}[3] || scalar @{$$self->{additional}} unless scalar @_;
-	carp 'header->arcount attribute is read-only' unless $warned++;
+	carp 'packet->header->arcount attribute is read-only'	    unless $warned++;
+	return;
 }
 
-sub zocount { &qdcount; }
-sub prcount { &ancount; }
-sub upcount { &nscount; }
-sub adcount { &arcount; }
+sub zocount { return &qdcount; }
+sub prcount { return &ancount; }
+sub upcount { return &nscount; }
+sub adcount { return &arcount; }
 
 
 =head1 EDNS Protocol Extensions
@@ -382,7 +391,7 @@ Gets or sets the EDNS DNSSEC OK flag.
 =cut
 
 sub do {
-	shift->_ednsflag( 0x8000, @_ );
+	return shift->_ednsflag( 0x8000, @_ );
 }
 
 
@@ -436,23 +445,23 @@ sub _dnsflag {
 	my $flag = shift;
 	for ( $$self->{status} ) {
 		my $set = $_ | $flag;
-		my $not = $set - $flag;
-		$_ = (shift) ? $set : $not if scalar @_;
-		return ( $_ & $flag ) ? 1 : 0;
+		$_ = (shift) ? $set : ( $set ^ $flag ) if scalar @_;
+		$flag &= $_;
 	}
+	return $flag ? 1 : 0;
 }
 
 
 sub _ednsflag {
 	my $self = shift;
 	my $flag = shift;
-	my $edns = $$self->edns->flags || 0;
-	return $flag & $edns ? 1 : 0 unless scalar @_;
-	my $set = $flag | $edns;
-	my $not = $set - $flag;
-	my $new = (shift) ? $set : $not;
-	$$self->edns->flags($new) unless $new == $edns;
-	return ( $new & $flag ) ? 1 : 0;
+	my $edns = $$self->edns;
+	for ( $edns->flags ) {
+		my $set = $_ | $flag;
+		$edns->flags( $_ = (shift) ? $set : ( $set ^ $flag ) ) if scalar @_;
+		$flag &= $_;
+	}
+	return $flag ? 1 : 0;
 }
 
 
@@ -468,7 +477,7 @@ Copyright (c)1997 Michael Fuhr.
 
 Portions Copyright (c)2002,2003 Chris Reinhardt.
 
-Portions Copyright (c)2012 Dick Franks.
+Portions Copyright (c)2012,2022 Dick Franks.
 
 All rights reserved.
 
@@ -477,7 +486,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
@@ -494,8 +503,8 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, L<Net::DNS::Packet>, L<Net::DNS::RR::OPT>
-RFC 1035 Section 4.1.1
+L<perl>, L<Net::DNS>, L<Net::DNS::Packet>, L<Net::DNS::RR::OPT>,
+L<RFC1035(4.1.1)|https://tools.ietf.org/html/rfc1035>
 
 =cut
 
